/* Script de creacin de las tablas set on /off */

CREATE TABLE CLIENTE (
NIF		VARCHAR2(9) PRIMARY KEY,
Nombre		VARCHAR2(20),
Apellido1	VARCHAR2(30),
Apellido2	VARCHAR2(30),
Ciudad		VARCHAR2(30),
Direccin	VARCHAR2(30),
CP		NUMBER(6),
Telfono	NUMBER(12),
Email		VARCHAR2(15),
Fecha_alta	Date,
Fecha_baja	Date
);

/* ****  creada tabla CLIENTE */

CREATE TABLE CARTERA (
Id_cartera	VARCHAR2(9) PRIMARY KEY,
NIF		VARCHAR2(9),
Fecha_Alta	Date,
Fecha_baja	Date,
CONSTRAINT NIF_CARTERA FOREIGN KEY (NIF) REFERENCES CLIENTE (NIF)
);

/* ****  creada tabla CARTERA */

CREATE TABLE MERCADO (

Id_mercado  	VARCHAR2(15) PRIMARY KEY,
Descripcin	VARCHAR2(100)
);

/* ****  creada tabla MERCADO */

CREATE TABLE ACCION (
Ticker		VARCHAR2(15) PRIMARY KEY,
Id_mercado	VARCHAR2(15),
Nombre		VARCHAR2(15),
CONSTRAINT Id_mercado_accion FOREIGN KEY (Id_mercado) REFERENCES MERCADO (Id_mercado)
);

/* ****  creada tabla ACCION  */


CREATE TABLE PRECIO_ACCION (

Ticker		VARCHAR2(15),
Fecha		Date,
Precio_euro	NUMBER(6,2),
Precio_dolar	NUMBER(6,2),

CONSTRAINT TF_precio_accion PRIMARY KEY (Ticker,Fecha),

CONSTRAINT Ticker_precio FOREIGN KEY (Ticker) REFERENCES ACCION (Ticker)
);


/* ****  creada tabla PRECIO_ACCION */

CREATE TABLE CARTERA_ACCION (
Id_cartera	VARCHAR2(9),
Ticker		VARCHAR2(15),
cantidad	NUMBER(10),
fecha_inicio		Date,
fecha_fin		Date,

CONSTRAINT PK_CARTERA_ACCION PRIMARY KEY (Id_cartera,Ticker,fecha),

CONSTRAINT FK1_CARTERA_ACCION FOREIGN KEY (Id_cartera) REFERENCES CARTERA (Id_cartera),

CONSTRAINT FK2_CARTERA_ACCION FOREIGN KEY (Ticker) REFERENCES ACCION (Ticker)

);


/* ****  creada tabla CARTERA_ACCION */

CREATE TABLE CAMBIO_DIVISA (
Tipo_divisa	VARCHAR2(10),
Fecha		Date,
Valor_Dlar	NUMBER(6,2),
CONSTRAINT PK_CAMBIO_DIVISA  PRIMARY KEY (Tipo_divisa,Fecha)


);

/* ****  creada tabla CAMBIO_DIVISA */

CREATE TABLE TRADING (
Id_trading		VARCHAR2(9),
Descripcin		VARCHAR2(50),
Fecha_creacin		Date,
CONSTRAINT PK_TRADING  PRIMARY KEY (Id_trading)

);


/* ****  creada tabla TRADING */

CREATE TABLE ORDEN_COMPRA_VENTA (

Id_orden	VARCHAR2(9),
NIF		VARCHAR2(9),
Ticker		VARCHAR2(15),
Id_trading		VARCHAR2(9),
Tipo_orden		VARCHAR2(10),  
Tipo_origen		VARCHAR2(12),  
Fecha_entrada_broker		Date,
Estado		VARCHAR2(10),
Fecha_ejec_mercado		Date,
Cantidad		NUMBER(10),
Precio_mnimo		NUMBER(8,5),
Precio_ejecucin		NUMBER(8,5),

CONSTRAINT PK_ORDEN_COMPRA_VENTA PRIMARY KEY (Id_orden),

CONSTRAINT FK1_ORDEN_COMPRA_VENTA FOREIGN KEY (NIF) REFERENCES CLIENTE (NIF),

CONSTRAINT FK2_ORDEN_COMPRA_VENTA FOREIGN KEY (Ticker) REFERENCES ACCION (Ticker),

CONSTRAINT FK3_ORDEN_COMPRA_VENTA FOREIGN KEY (Id_trading) REFERENCES TRADING (Id_trading),

CONSTRAINT C1_ORDEN_COMPRA_VENTA CHECK (Tipo_orden IN ('VENTA','COMPRA')),

CONSTRAINT C2_ORDEN_COMPRA_VENTA CHECK (Tipo_origen IN ('MANUAL','AUTOMATICA')),

CONSTRAINT C3_ORDEN_COMPRA_VENTA CHECK (Estado IN ('EJECUTADA','CANCELADA','PENDIENTE'))


);

/* ****  creada tabla ORDEN_COMPRA_VENTA */






 CREATE TABLE TRADING_CLIENTE(
Id_trading		 VARCHAR2(9),
NIF		         VARCHAR2(9),
fecha                    Date,

CONSTRAINT PK_TRADING_CLIENTE PRIMARY KEY (Id_trading,NIF),

CONSTRAINT FK1_TRADING_CLIENTE FOREIGN KEY (NIF) REFERENCES CLIENTE (NIF),

CONSTRAINT FK2_TRADING_CLIENTE FOREIGN KEY (Id_trading) REFERENCES TRADING (Id_trading)

);


/* ****  creada tabla TRADING_CLIENTE */




CREATE TABLE EST_BENEFICIO (
ID		NUMBER(6,2),
NIF		VARCHAR2(9),
Beneficio_neto	NUMBER(10,2),
Porcentaje	NUMBER(6,2),
Mes		Date,

CONSTRAINT PK_EST_BENEFICIO  PRIMARY KEY (ID),

CONSTRAINT FK1_EST_BENEFICIO FOREIGN KEY (NIF) REFERENCES CLIENTE (NIF)

);


/* ****  creada tabla EST_BENEFICIO */


CREATE TABLE EST_PENDIENTES (
ID		NUMBER(6,2),
Ticker		VARCHAR2(15),
Acciones_dispo_broker	NUMBER(9),
Num_clientes		NUMBER(9),

CONSTRAINT PK_EST_PENDIENTES PRIMARY KEY (ID),

CONSTRAINT FK1_EST_PENDIENTES FOREIGN KEY (Ticker) REFERENCES ACCION (Ticker)

);


/* ****  creada tabla EST_PENDIENTES */

CREATE TABLE EST_CARTERA_MAYOR (
ID		NUMBER(6,2),
NIF		VARCHAR2(9),
Valor		VARCHAR2(10),
Fecha		Date,

CONSTRAINT PK_EST_CARTERA_MAYOR PRIMARY KEY (ID),

CONSTRAINT FK1_EST_CARTERA_MAYOR FOREIGN KEY (NIF) REFERENCES CLIENTE (NIF)

);

/* ****  creada tabla EST_CARTERA_MAYOR */

CREATE TABLE EST_NEGOCIADA (
ID		NUMBER(6,2),
Ticker		VARCHAR2(10),
Volumen		NUMBER(10),
Tipo		VARCHAR2(10),   
Fecha		Date,

CONSTRAINT PK_EST_NEGOCIADA PRIMARY KEY (ID),

CONSTRAINT FK1_EST_NEGOCIADA FOREIGN KEY (Ticker) REFERENCES ACCION (Ticker),

CONSTRAINT C1_EST_NEGOCIADA CHECK (Tipo IN ('COMPRADA','VENDIDA'))

);

/* ****  creada tabla EST_NEGOCIADA */


CREATE TABLE LOGS (

Id_log			VARCHAR2(10) PRIMARY KEY,
Fecha_ejecucin		Date,	
Func_proc		VARCHAR2(30),	
Par_entrada		VARCHAR2(200),	
Par_salida		VARCHAR2(200),	
Estado			VARCHAR2(10),
descripcion		VARCHAR2(30)


CONSTRAINT C1_LOGS CHECK (Estado IN ('ERROR','OK'))

);


/* ****  creada tabla LOGS */


