
/* Paquete  PAQ_CARTERA_ACCION  para alta, baja y modificacin de datos de las acciones de las carteras de los clientes.
   Tablas afectadas: CARTERA (id_cartera, NIF,Fecha_alta, Fecha_baja),
                     CARTERA_ACCION (Id_cartera, ticker, cantidad, fecha_inicio, fecha_fin)
   Formado por los procedimientos: IN_CARTERA_ACCION, DEL_CARTERA_ACCION, UP_CARTERA_ACCION

*/



CREATE OR REPLACE PACKAGE PAQ_CARTERA_ACCION AS 

   PROCEDURE IN_CARTERA_ACCION ( par_Id_cartera  IN CARTERA_ACCION.Id_cartera%TYPE,
                                   par_ticker IN CARTERA_ACCION.ticker%TYPE,
			           par_cantidad IN CARTERA_ACCION.cantidad%TYPE,
                                   par_fecha_inicio IN CARTERA_ACCION.fecha_inicio%TYPE);

        

  PROCEDURE DEL_CARTERA_ACCION (par_Id_cartera  IN CARTERA_ACCION.Id_cartera%TYPE,
                                  par_ticker IN CARTERA_ACCION.ticker%TYPE,
                                  par_Fecha_fin IN CARTERA_ACCION.Fecha_fin%TYPE);

        

  PROCEDURE UP_CARTERA_ACCION (par_Id_cartera  IN CARTERA_ACCION.Id_cartera%TYPE,
                                 par_ticker IN CARTERA_ACCION.ticker%TYPE,
                                 par_cantidad IN CARTERA_ACCION.cantidad%TYPE,
                                 par_Fecha_inicio IN CARTERA_ACCION.Fecha_inicio%TYPE,
			         par_Fecha_fin IN CARTERA_ACCION.Fecha_fin%TYPE);


END PAQ_CARTERA_ACCION;

/



CREATE OR REPLACE PACKAGE BODY PAQ_CARTERA AS

/*
   NOMBRE:      IN_CARTERA_ACCION
   DESCRIPCIN: Procedimiento para altas de acciones en las CARTERAS
                  
   PARMETROS:
       INPUT:  Obligatorios: par_Id_cartera, par_ticker, par_cantidad, par_fecha_inicio
       OUTPUT:  
   PRECONDICION: Los campos obligatorios tengan valores correctos.
   POSTCONDICION: Se da de alta las acciones en la cartera de inversin del cliente
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: 'El Id_cartera no puede ser nulo'
                'ERROR: 'El ticker no puede ser nulo'
                'ERROR: 'La cantidad no puede ser nula'
                'ERROR: 'La fecha_inicio no puede ser nula'
                'ERROR: (SQLERRM)'
*/

 PROCEDURE IN_CARTERA_ACCION (par_Id_cartera  IN CARTERA_ACCION.Id_cartera%TYPE,
		                par_ticker IN CARTERA_ACCION.ticker%TYPE,
		                par_cantidad IN CARTERA_ACCION.cantidad%TYPE,
                                par_fecha_inicio IN CARTERA_ACCION.fecha_inicio%TYPE) 
IS


     error EXCEPTION;
     numero       NUMBER;
     mensaje       VARCHAR2(200);
     PAR_IN        VARCHAR2(500);
     RSP           VARCHAR2(500);
     v_usuario	   VARCHAR2(20);


     BEGIN

         SELECT USER into v_usuario
         FROM DUAL;


     /* Hay que comprobar que los parmetros de entrada sean correctos: */

     PAR_IN := par_Id_cartera || ' ' || par_ticker || ' ' ||  par_cantidad || par_fecha_inicio;     
     
    IF par_Id_cartera IS NULL 
     THEN RSP :='El Id_cartera no puede ser nulo'; RAISE error;
      
     END IF;


    IF par_ticker IS NULL 
     THEN RSP :='El ticker no puede ser nulo'; RAISE error;
      
     END IF;

    
    IF par_cantidad IS NULL 
     THEN RSP :='La cantidad no puede ser nula'; RAISE error;
      
     END IF;

    IF par_fecha_inicio IS NULL 
     THEN RSP :='La fecha_inicio no puede ser nula'; RAISE error;
      
     END IF;
    
      -- se inserta en CARTERA_ACCIONES los valores de los parmetros
      INSERT INTO CARTERA_ACCION (Id_cartera,ticker,cantidad,Fecha_inicio) 
      VALUES (par_Id_cartera,par_ticker,par_cantidad, par_Fecha_inicio);

      -- se almacena en la tabla LOGS lo que se ejecuta para comprobacin.
      INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado) 
         VALUES(SEQ_Id_log.nextval,SYSDATE,'PAQ_CARTERA_ACCIONES.IN_CARTERA_ACCIONES',PAR_IN,'OK',RSP);
       
     
     EXCEPTION

        WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CARTERA_ACCIONES.IN_CARTERA_ACCIONES',PAR_IN,RSP,'ERROR',v_usuario);

         
        WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         RSP:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CARTERA_ACCIONES.IN_CARTERA_ACCIONES',PAR_IN,RSP,'ERROR',v_usuario);

   
  END IN_CARTERA;


/* 
   NOMBRE: DEL_CARTERA_ACCION
   DESCRIPCIN: Procedimiento para dar baja carteras de clientes, hay que modificar la tabla e insertar fecha_baja
                  
   PARMETROS:
       INPUT:  Obligatorios: par_Id_cartera, 
       OUTPUT:  res VARCHAR2(300)
   PRECONDICION: Los campos obligatorios tengan valores correctos.
   POSTCONDICION: Se da de baja el ticker de la cartera del cliente.
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: 'El identificador de cartera no puede ser nulo';
                'ERROR: 'EL ticker no puede ser nulo';
                'ERROR: (SQLERRM)'
*/

 
 PROCEDURE DEL_CARTERA_ACCION (par_Id_cartera  IN CARTERA_ACCION.Id_cartera%TYPE,
                                  par_ticker IN CARTERA_ACCION.ticker%TYPE,
                                  par_Fecha_fin IN CARTERA_ACCION.Fecha_fin%TYPE) 
IS
     error EXCEPTION;
     numero        NUMBER;
     mensaje       VARCHAR2(200);
     v_comprobar   CARTERA.Id_cartera%TYPE;
     PAR_IN        VARCHAR2(500);
     RSP           VARCHAR2(500);
     v_contar	   NUMBER;
     v_usuario     VARCHAR2(20);

     BEGIN

         SELECT USER into v_usuario
         FROM DUAL;


     /* Hay que comprobar que los parmetros de entrada sean correctos: */

     PAR_IN := par_Id_cartera || ' ' || par_ticker || ' ' || par_Fecha_baja;     
     
   
    IF par_Id_cartera IS NULL 
     THEN  RSP :='El identificador de cartera no puede ser nulo'; RAISE error;
      END IF;

    IF par_ticker IS NULL 
      THEN RSP :='EL ticker no puede ser nulo'; RAISE error;
      END IF;

     IF par_Fecha_fin IS NULL 
     THEN RSP :='La fecha_fin no puede ser nula'; RAISE error;
      
     END IF;


   -- Hay que comprobar que el identificador de cartera exista.
    BEGIN
         SELECT fecha_baja INTO v_comprobar
         FROM CARTERA
         WHERE id_cartera = par_Id_cartera;
   EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El identificador de cartera no existe';
           RAISE error;
   END;

                UPDATE CARTERA_ACCION SET fecha_fin=par_fecha_fin
                 WHERE id_cartera = par_id_cartera and ticker = par_ticker;


                -- para comprobar lo que se ejecuta.
                 INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
                 VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.DEL_CARTERA',PAR_IN,RSP,'OK',v_usuario);
       
           
     EXCEPTION

      WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.DEL_CARTERA',PAR_IN,RSP,'ERROR',v_usuario);

         
        WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         RSP:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PK_CARTERA.DEL_CARTERA',PAR_IN,RSP,'ERROR',v_usuario);

   
  END DEL_CARTERA;


  
/* Procedimiento para modificar en CARTERA_ACCION e insertar los valores de los parmetros
  
   PARAMETROS:
        INPUT: par_Id_cartera  IN CARTERA_ACCION.Id_cartera%TYPE,
                                 par_ticker IN CARTERA_ACCION.ticker%TYPE,
                                 par_cantidad IN CARTERA_ACCION.cantidad%TYPE,
                                 par_Fecha_inicio IN CARTERA_ACCION.Fecha_inicio%TYPE,
			         par_Fecha_fin IN CARTERA_ACCION.Fecha_fin%TYPE

        OUTPUT:  RSP 
   PRECONDICION: Los campos obligatorios tengan valores correctos.
   POSTCONDICION: Se modifican los datos de las acciones de la cartera del cliente.
   EXCEPTION:   'ERROR: El id_cartera del cliente no puede ser nulo
                
                'ERROR: (SQLERRM)'
*/

  PROCEDURE UP_CARTERA_ACCION (par_Id_cartera  IN CARTERA_ACCION.Id_cartera%TYPE,
                                 par_ticker IN CARTERA_ACCION.ticker%TYPE,
                                 par_cantidad IN CARTERA_ACCION.cantidad%TYPE,
                                 par_Fecha_inicio IN CARTERA_ACCION.Fecha_inicio%TYPE,
			         par_Fecha_fin IN CARTERA_ACCION.Fecha_fin%TYPE)
    IS

   BEGIN 

   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   RSP           VARCHAR2(500);
   v_NIF         CARTERA.NIF%TYPE;
   v_id_cartera  CARTERA.id_cartera%TYPE;
   v_fecha_baja  CARTERA.id_fecha_baja%TYPE;
   error 	 EXCEPTION;
   v_usuario     VARCHAR2(30);

         SELECT USER into v_usuario
         FROM DUAL;

     PAR_IN :=  par_Id_cartera || ' ' || par_ticker || ' ' || par_cantidad || ' ' || par_Fecha_inicio || ' ' || par_Fecha_fin;


  -- Hay que comprobar que los parmetros de entrada sean correctos: 
   
     -- se comprueba que el par_Id_cartera no sea nulo.
     
     IF par_Id_cartera IS NULL THEN 
         RSP := 'ERROR: El Id_cartera no puede ser nulo';
        RAISE error; 
     END IF;

     -- se comprueba que el par_ticker no sea nulo.
     
     IF par_ticker IS NULL THEN 
        RSP := 'ERROR: El ticker no puede ser nulo';
       RAISE error;
     END IF;

      -- se comprueba que el par_cantidad no sea nulo.
     
     IF par_cantidad IS NULL THEN 
        RSP := 'ERROR: La cantidad no puede ser nula';
       RAISE error;
     END IF;


        
         -- Hay que comprobar que el Id_cartera exista en la base de datos.
         BEGIN
 	         SELECT Id_cartera INTO v_id_cartera
                 FROM CARTERA
                 WHERE Id_cartera = par_Id_cartera;
         EXCEPTION
                WHEN NO_DATA_FOUND THEN
                  RSP := 'ERROR: El id_cartera no existe en la base de datos';
                  RAISE error;
         END;

         
               -- se atualiza la tabla CARTERA_ACCION con los datos de los parmetros.

              UPDATE CARTERA_ACCION SET ticker=par_ticker,cantidad=par_cantidad,Fecha_inicio=par_Fecha_inicio,
                     Fecha_fin=par_Fecha_fin
              WHERE Id_cartera = par_Id_cartera and ticker=par_ticker;


               -- para comprobar lo que se ejecuta.
               INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
                 VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CARTERA_ACCION.UP_CARTERA_ACCIONES',PAR_IN,RSP,'OK',v_usuario);

   EXCEPTION

      WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CARTERA_ACCION.UP_CARTERA_ACCIONES',PAR_IN,RSP,'ERROR',v_usuario);

         
        WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         RSP:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CARTERA_ACCION.UP_CARTERA_ACCIONES',PAR_IN,RSP,'ERROR',v_usuario);
   END;

  
END PAQ_CARTERA_ACCION;

