/* Paquete  PAQ_CLIENTE  para alta, baja y modificacin de datos de los clientes.
   Tablas afectadas: CLIENTE, LOGS
   Formado por los procedimientos: IN_CLIENTE, DEL_CLIENTE, UP_CLIENTE

*/

CREATE OR REPLACE PACKAGE PAQ_CLIENTE AS 

   PROCEDURE IN_CLIENTE (par_NIF  IN CLIENTE.NIF%TYPE,
                          par_Nombre  IN CLIENTE.Nombre%TYPE,
                          par_Apellido1 IN CLIENTE.Apellido1%TYPE,
			  par_Apellido2 IN CLIENTE.Apellido2%TYPE,
                          par_Ciudad IN CLIENTE.Ciudad%TYPE,
                          par_Direccin IN CLIENTE.Direccin%TYPE,
                          par_Telfono IN CLIENTE.Telfono%TYPE,
                          par_Email IN CLIENTE.Email%TYPE,
                          par_Fecha_alta IN CLIENTE.Fecha_alta%TYPE);

  PROCEDURE DEL_CLIENTE (par_NIF IN CLIENTE.NIF%TYPE, par_fecha IN CLIENTE.fecha_baja%TYPE);

  PROCEDURE UP_CLIENTE (par_NIF  IN CLIENTE.NIF%TYPE,
                          par_Nombre  IN CLIENTE.Nombre%TYPE,
                          par_Apellido1 IN CLIENTE.Apellido1%TYPE,
			  par_Apellido2 IN CLIENTE.Apellido2%TYPE,
                          par_Ciudad IN CLIENTE.Ciudad%TYPE,
                          par_Direccin IN CLIENTE.Direccin%TYPE,
                          par_Telfono IN CLIENTE.Telfono%TYPE,
                          par_Email IN CLIENTE.Email%TYPE,
                          par_Fecha_alta IN CLIENTE.Fecha_alta%TYPE,
                          par_Fecha_baja IN CLIENTE.Fecha_baja%TYPE);

END PAQ_CLIENTE;
/


CREATE OR REPLACE PACKAGE BODY PAQ_CLIENTE AS



/* Procedimiento para dar de alta a CLIENTES, hay que insertar en la tabla CLIENTE e insertar los valores de los parmetros 
  
   PARAMETROS:
        INPUT: par_NIF IN CLIENTE.NIF%TYPE,
                          par_Nombre  IN CLIENTE.Nombre%TYPE,
                          par_Apellido1 IN CLIENTE.Apellido1%TYPE,
			  par_Apellido2 IN CLIENTE.Apellido2%TYPE,
                          par_Ciudad IN CLIENTE.Ciudad%TYPE,
                          par_Direccin IN CLIENTE.Direccin%TYPE,
                          par_Telfono IN CLIENTE.Telfono%TYPE,
                          par_Email IN CLIENTE.Email%TYPE,
                          par_Fecha_alta IN CLIENTE.Fecha_alta%TYPE

        OUTPUT:  RSP 
   
   POSTCONDICION: Se insertan los datos del cliente.
   EXCEPTION:   'ERROR: El NIF del cliente no puede ser nulo
                'ERROR: El Apellido1 del cliente no puede ser nulo
                'ERROR: El Apellido2 del cliente no puede ser nulo
                'ERROR: La fecha de alta no puede ser nula
                'ERROR: (SQLERRM)'
*/


     PROCEDURE IN_CLIENTE (par_NIF IN CLIENTE.NIF%TYPE,
                          par_Nombre  IN CLIENTE.Nombre%TYPE,
                          par_Apellido1 IN CLIENTE.Apellido1%TYPE,
			  par_Apellido2 IN CLIENTE.Apellido2%TYPE,
                          par_Ciudad IN CLIENTE.Ciudad%TYPE,
                          par_Direccin IN CLIENTE.Direccin%TYPE,
                          par_Telfono IN CLIENTE.Telfono%TYPE,
                          par_Email IN CLIENTE.Email%TYPE,
                          par_Fecha_alta IN CLIENTE.Fecha_alta%TYPE
                          ) IS
     error EXCEPTION;
     numero       NUMBER;
     mensaje       VARCHAR2(200);
     PAR_IN 	   VARCHAR2(500);
     RSP           VARCHAR2(500);   
     v_usuario     VARCHA2(20);

     BEGIN

        SELECT USER into v_usuario
         FROM DUAL;

      PAR_IN:= par_NIF || ' ' || par_Nombre || ' ' || par_Apellido1 || ' ' || par_Apellido2 || ' ' || par_Ciudad || ' ' || 

 par_Direccin || ' ' || par_Telfono || ' ' || par_Email || ' ' || par_Fecha_alta;


     /* Hay que comprobar que los parmetros de entrada sean correctos: */


     IF par_NIF IS NULL 
     THEN  RAISE error;
      res :='El NIF no puede ser nulo';
     END IF;

     IF par_Nombre IS NULL 
     THEN  RAISE error;
      res :='El Nombre no puede ser nulo';
     END IF;

     IF par_Apellido1 IS NULL 
     THEN  RAISE error;
      res :='El Apellido1 no puede ser nulo';
     END IF;
    
     IF par_Apellido2 IS NULL 
     THEN  RAISE error;
      res :='El Apellido2 no puede ser nulo';
     END IF;

     IF par_Fecha_alta IS NULL 
     THEN  RAISE error;
      res :='La fecha de alta no puede ser nula';
     END IF;

        -- se inserta en la tabla CLIENTE los valores de los parmetros.
      INSERT INTO CLIENTE (NIF,Nombre,Apellido1,Apellido2,Ciudad,Direccin,Ciudad,Telfono,Email,Fecha_alta) 
      VALUES (par_NIF,par_Nombre,par_Apellido1,par_Apellido2,par_Ciudad,par_Direccin,par_Telfono,par_Email,par_Fecha_alta);
     

        -- se almacena en la tabla LOGS para comprobar lo que se ha ejecutado.
       INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
        VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PR_CIERRE_DIARIO',PAR_IN,RSP,'OK',v_usuario);


     EXCEPTION

      WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CLIENTE.IN_CLIENTE',PAR_IN,RSP,'ERROR',v_usuario);

         
      WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         Res:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CLIENTE.IN_CLIENTE,PAR_IN,RSP,'ERROR',v_usuario);

   
  END IN_CLIENTE;



/* Procedimiento para modificar en CLIENTES, hay que modificar la tabla CLIENTE e insertar los valores de los parmetros */
  
   PARAMETROS:
        INPUT: par_NIF  IN CLIENTE.NIF%TYPE,
                          par_Nombre  IN CLIENTE.Nombre%TYPE,
                          par_Apellido1 IN CLIENTE.Apellido1%TYPE,
			  par_Apellido2 IN CLIENTE.Apellido2%TYPE,
                          par_Ciudad IN CLIENTE.Ciudad%TYPE,
                          par_Direccin IN CLIENTE.Direccin%TYPE,
                          par_Telfono IN CLIENTE.Telfono%TYPE,
                          par_Email IN CLIENTE.Email%TYPE,
                          par_Fecha_alta IN CLIENTE.Fecha_alta%TYPE,
                          par_Fecha_baja IN CLIENTE.Fecha_baja%TYPE

        OUTPUT:  RSP 
   PRECONDICION: Debe existir el cliente
   POSTCONDICION: Se modifican los datos del cliente.
   EXCEPTION:   'ERROR: El NIF del cliente no puede ser nulo
                'ERROR: El cliente no existe
                'ERROR: El cliente tiene una cartera de acciones activa.
                'ERROR: (SQLERRM)'
*/

  PROCEDURE UP_CLIENTE (par_NIF  IN CLIENTE.NIF%TYPE,
                          par_Nombre  IN CLIENTE.Nombre%TYPE,
                          par_Apellido1 IN CLIENTE.Apellido1%TYPE,
			  par_Apellido2 IN CLIENTE.Apellido2%TYPE,
                          par_Ciudad IN CLIENTE.Ciudad%TYPE,
                          par_Direccin IN CLIENTE.Direccin%TYPE,
                          par_Telfono IN CLIENTE.Telfono%TYPE,
                          par_Email IN CLIENTE.Email%TYPE,
                          par_Fecha_alta IN CLIENTE.Fecha_alta%TYPE,
                          par_Fecha_baja IN CLIENTE.Fecha_baja%TYPE);
    IS

   BEGIN

   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   RSP           VARCHAR2(500);
   v_NIF         CARTERA.NIF%TYPE;
   v_id_cartera  CARTERA.id_cartera%TYPE;
   v_fecha_baja  CARTERA.id_fecha_baja%TYPE;
   error	 EXCEPTION;
   v_usuario     VARCHAR2(20);

         SELECT USER into v_usuario
         FROM DUAL;
   

     PAR_IN := par_NIF || ' ' || par_Nombre || ' ' || par_Apellido1  || ' ' || par_Apellido2 
     par_Ciudad || ' ' || par_Direccin || ' ' || par_Telfono || ' ' || par_Email 
     || ' ' || par_Fecha_alta || ' ' || par_Fecha_baja;


  -- Hay que comprobar que los parmetros de entrada sean correctos: 
   
     -- se comprueba que el NIF no sea nulo.
     RSP := 'ERROR: El NIF no puede ser nulo';
     IF par_NIF IS NULL THEN  RAISE error; END IF;

     
         -- Hay que comprobar que el NIF exista en la base de datos.
         BEGIN
 	         SELECT NIF INTO v_NIF
                 FROM cliente
                 WHERE NIF = v_NIF;
         EXCEPTION
                WHEN NO_DATA_FOUND THEN
                  RSP := 'ERROR: El cliente no existe en la base de datos';
                  RAISE error;
         END;

        -- Hay que tener cuidado porque no es conveniente modificar la fecha de baja a un cliente que tenga activa su cartera de acciones.

         BEGIN
 	         SELECT id_cartera, fecha_baja INTO v_id_cartera, v_fecha_baja
                 FROM cliente
                 WHERE NIF = v_NIF;

              RSP := 'ERROR: El cliente tiene activa la cartera ' || v_id_cartera ;
             IF v_fecha_baja is NULL THEN RAISE error; END IF;
 

         EXCEPTION
                WHEN NO_DATA_FOUND THEN
                  RSP := 'ERROR: Al consultar la fecha de baja de una cartera';
                  RAISE error;
         END;

           -- Actualizo los valores de la tabla de CLIENTE segn los parmetros recibidos.
           UPDATE CLIENTE SET nombre=par_Nombre, Apellido1=par_Apellido1, Apellido2=par_Apellido2,
                          Ciudad=par_Ciudad,Direccin=par_Direccin, Telfono=par_Telfono,
                          Email=par_Email, Fecha_alta=par_Fecha_alta, Fecha_baja=par_Fecha_baja
            WHERE NIF=par_NIF;         
 
           -- Inserto en la tabla LOGS para controlar lo que se ejecuta.
           INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CLIENTE.UP_CLIENTE',PAR_IN,RSP,'OK',v_usuario);
   EXCEPTION

      WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CLIENTE.UP_CLIENTE',PAR_IN,RSP,'ERROR',v_usuario);

         
      WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         Res:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CLIENTE.UP_CLIENTE,PAR_IN,RSP,'ERROR',v_usuario);

   END;



  /* Procedimiento para baja de CLIENTES, hay que modificar la tabla CLIENTE e insertar fecha_baja */
  Pero cuidado, porque si damos de baja a un cliente, si tiene carteras habr que dar de baja tambin stas.*/

  Se considera que no se realice el borrado fsico con DELETE FROM, ya que se considera de gran utilidad la informacin 
   del cliente aunque est de baja. Por lo tanto se actualiza la fecha_baja segn el parmetro.


   PARAMETROS:
        INPUT: 

        OUTPUT:  RSP 
   PRECONDICION: Debe existir el cliente
   POSTCONDICION: Se da de baja el cliente
   EXCEPTION:   'ERROR: El NIF del cliente no puede ser nulo
                'ERROR: El cliente no existe
                'ERROR: El cliente tiene una cartera de acciones activa.
                'ERROR: (SQLERRM)'
*/

  PROCEDURE DEL_CLIENTE (par_NIF IN CLIENTE.NIF%TYPE,par_fecha CLIENTE.fecha_baja%TYPE)
    IS

   BEGIN

   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   RSP           VARCHAR2(500);
   v_NIF         CARTERA.NIF%TYPE;
   v_id_cartera  CARTERA.id_cartera%TYPE;
   v_fecha_baja  CARTERA.id_fecha_baja%TYPE;
   error	 EXCEPTION;
   v_usuario     VARCHAR2(30);
   

     PAR_IN := par_NIF || ' ' || par_fecha;

         SELECT USER into v_usuario
         FROM DUAL;


  -- Hay que comprobar que los parmetros de entrada sean correctos: 
   
     -- se comprueba que el NIF no sea nulo.
     RSP := 'ERROR: El NIF no puede ser nulo';
     IF par_NIF IS NULL THEN  RAISE error; END IF;

     -- se comprueba que la fecha de baja no sea nula.
     RSP := 'ERROR: La fecha de baja del cliente no puede ser nula';
     IF par_fecha IS NULL THEN  RAISE error; END IF;

         -- Hay que comprobar que el NIF exista en la base de datos.
         BEGIN
 	         SELECT NIF INTO v_NIF
                 FROM cliente
                 WHERE NIF = v_NIF;
         EXCEPTION
                WHEN NO_DATA_FOUND THEN
                  RSP := 'ERROR: El cliente no existe en la base de datos';
                  RAISE error;
         END;

        -- Hay que tener cuidado porque no es conveniente dar de baja a un cliente que tenga activa su cartera de acciones.

         BEGIN
 	         SELECT id_cartera, fecha_baja INTO v_id_cartera, v_fecha_baja
                 FROM CARTERA
                 WHERE NIF = v_NIF;

              RSP := 'ERROR: El cliente tiene activa la cartera ' || v_id_cartera ;
             IF v_fecha_baja is NULL THEN RAISE error; END IF;
 

         EXCEPTION
                WHEN NO_DATA_FOUND THEN
                  RSP := 'ERROR: Al consultar la fecha de baja de una cartera';
                  RAISE error;
         END;

           -- Actualizo el valor de la fecha de baja del cliente.
           UPDATE CLIENTE fecha_baja= par_fecha_baja WHERE NIF=var_NIF;         
 
           -- Inserto en la tabla LOGS para controlar lo que se ejecuta.

           INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CLIENTE.DEL_CLIENTE',PAR_IN,RSP,'OK',v_usuario);


   EXCEPTION

      WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CLIENTE.DEL_CLIENTE',PAR_IN,RSP,'ERROR',v_usuario);

         
      WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         Res:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_CLIENTE.DEL_CLIENTE,PAR_IN,RSP,'ERROR',v_usuario);
   END;





END PAQ_CLIENTE;
