/* Paquete  PAQ_ORDEN_COMPRA_VENTA  para alta, baja y modificacin de datos de los clientes.
  
   Formado por los procedimientos: IN_ORDEN_COMPRA_VENTA, DEL_ORDEN_COMPRA_VENTA, UP_ORDEN_COMPRA_VENTA

*/

CREATE OR REPLACE PACKAGE PAQ_ORDEN_COMPRA_VENTA AS 

   PROCEDURE IN_ORDEN_COMPRA_VENTA (
                                    par_NIF IN ORDEN_COMPRA_VENTA.NIF%TYPE,
                                    par_Ticker IN ORDEN_COMPRA_VENTA.Ticker%TYPE,
                                    par_Tipo_orden IN ORDEN_COMPRA_VENTA.Tipo_orden%TYPE,
                                    par_Tipo_origen IN ORDEN_COMPRA_VENTA.Tipo_origen%TYPE,
                                    par_Fecha_entrada_broker IN ORDEN_COMPRA_VENTA.Fecha_entrada_broker%TYPE,
                                    par_Estado IN ORDEN_COMPRA_VENTA.Estado%TYPE, 
                                    par_Fecha_ejec_mercado IN ORDEN_COMPRA_VENTA.Fecha_ejec_mercado%TYPE,
                                    par_Cantidad IN ORDEN_COMPRA_VENTA.Cantidad%TYPE,
                                    par_Precio_minimo IN ORDEN_COMPRA_VENTA.Precio_mnimo%TYPE,
                                    par_Precio_ejecucion IN ORDEN_COMPRA_VENTA.Precio_ejecucin%TYPE);



  PROCEDURE DEL_ORDEN_COMPRA_VENTA (par_Id_orden IN ORDEN_COMPRA_VENTA.Id_orden%TYPE);

  PROCEDURE UP_ORDEN_COMPRA_VENTA (par_Id_orden IN ORDEN_COMPRA_VENTA.Id_orden%TYPE,
                                    par_Tipo_orden IN ORDEN_COMPRA_VENTA.Tipo_orden%TYPE,
                                    par_Tipo_origen IN ORDEN_COMPRA_VENTA.Tipo_origen%TYPE,
                                    par_Fecha_entrada_broker IN ORDEN_COMPRA_VENTA.Fecha_entrada_broker%TYPE,
                                    par_Estado IN ORDEN_COMPRA_VENTA.Estado%TYPE, 
                                    par_Fecha_ejec_mercado IN ORDEN_COMPRA_VENTA.Fecha_ejec_mercado%TYPE,
                                    par_Cantidad IN ORDEN_COMPRA_VENTA.Cantidad%TYPE,
                                    par_Precio_minimo IN ORDEN_COMPRA_VENTA.Precio_mnimo%TYPE,
                                    par_Precio_ejecucion IN ORDEN_COMPRA_VENTA.Precio_ejecucin%TYPE);

END PAQ_ORDEN_COMPRA_VENTA;
/



CREATE OR REPLACE PACKAGE BODY PAQ_ORDEN_COMPRA_VENTA AS


/* Procedimiento para dar de alta rdenes de compra/venta, hay que insertar en la tabla ORDEN_COMPRA_VENTA e insertar los valores de los parmetros 
  
   PARAMETROS:
        INPUT: par_NIF IN ORDEN_COMPRA_VENTA.NIF%TYPE,
               par_Ticker IN ORDEN_COMPRA_VENTA.Ticker%TYPE,
               par_id_trading IN ORDEN_COMPRA_VENTA.id_trading%TYPE,
               par_Tipo_orden IN ORDEN_COMPRA_VENTA.Tipo_orden%TYPE,
               par_Tipo_origen IN ORDEN_COMPRA_VENTA.Tipo_origen%TYPE,
               par_Fecha_entrada_broker IN ORDEN_COMPRA_VENTA.Fecha_entrada_broker%TYPE,
               par_Estado IN ORDEN_COMPRA_VENTA.Estado%TYPE, 
               par_Fecha_ejec_mercado IN ORDEN_COMPRA_VENTA.Fecha_ejec_mercado%TYPE,
               par_Cantidad IN ORDEN_COMPRA_VENTA.Cantidad%TYPE,
               par_Precio_minimo IN ORDEN_COMPRA_VENTA.Precio_mnimo%TYPE,
               par_Precio_ejecucion IN ORDEN_COMPRA_VENTA.Precio_ejecucin%TYPE);


        OUTPUT:  RSP 
   PRECONDICION: El valor de los parmetros es correcto.
   POSTCONDICION: Se insertan la rden de compra/venta
   EXCEPTION:   'ERROR: 'El NIF no puede ser nulo';
                'ERROR: 'El ticker no puede ser nulo';
                'ERROR: 'El tipo de orden no puede ser nulo';
                'ERROR: 'El tipo de origen no puede ser nulo';
                'ERROR: 'El estado no puede ser nulo';
                'ERROR: 'La cantidad no puede ser nula';
                'ERROR: (SQLERRM)'
*/

     PROCEDURE IN_ORDEN_COMPRA_VENTA (
                                    par_NIF IN ORDEN_COMPRA_VENTA.NIF%TYPE,
                                    par_Ticker IN ORDEN_COMPRA_VENTA.Ticker%TYPE,
                                    par_Tipo_orden IN ORDEN_COMPRA_VENTA.Tipo_orden%TYPE,
                                    par_Tipo_origen IN ORDEN_COMPRA_VENTA.Tipo_origen%TYPE,
                                    par_Fecha_entrada_broker IN ORDEN_COMPRA_VENTA.Fecha_entrada_broker%TYPE,
                                    par_Estado IN ORDEN_COMPRA_VENTA.Estado%TYPE, 
                                    par_Fecha_ejec_mercado IN ORDEN_COMPRA_VENTA.Fecha_ejec_mercado%TYPE,
                                    par_Cantidad IN ORDEN_COMPRA_VENTA.Cantidad%TYPE,
                                    par_Precio_minimo IN ORDEN_COMPRA_VENTA.Precio_mnimo%TYPE,
                                    par_Precio_ejecucion IN ORDEN_COMPRA_VENTA.Precio_ejecucin%TYPE) IS
     error         EXCEPTION;
     numero        NUMBER;
     mensaje       VARCHAR2(200);
     PAR_IN        VARCHAR2(500);
     RSP           VARCHAR2(500);
     v_cartera     CARTERA.NIF%TYPE;
     v_cantidad    NUMBER;
      v_usuario     VARCHAR2(30);

     BEGIN

         SELECT USER into v_usuario
         FROM DUAL;

        PAR_IN := par_NIF || ' ' || par_Ticker || ' ' || par_Tipo_orden || ' ' || par_Tipo_origen || ' ' || par_Fecha_entrada_broker
|| ' ' || par_Estado || ' ' || par_Fecha_ejec_mercado || ' ' || par_Cantidad || ' ' || par_Precio_minimo;


     /* Hay que comprobar que los parmetros obligatorios de entrada sean correctos: */

     -- se comprueba el NIF
     IF par_NIF IS NULL 
     THEN  RAISE error;
      RSP :='El NIF no puede ser nulo';
     END IF;

    -- se comprueba el Ticker
    IF par_Ticker IS NULL 
     THEN  RAISE error;
      RSP :='El ticker no puede ser nulo';
     END IF;

    -- se comprueba el Tipo_orden
    IF par_Tipo_orden IS NULL 
     THEN  RAISE error;
      RSP :='El tipo de orden no puede ser nulo';
     END IF;

   -- se comprueba el Tipo_origen
    IF par_Tipo_origen IS NULL 
     THEN  RAISE error;
      RSP :='El tipo de origen no puede ser nulo';
     END IF;
   
    -- se comprueba el par_Estado
    IF par_Estado IS NULL 
     THEN  RAISE error;
      RSP :='El estado no puede ser nulo';
     END IF;

   -- se comprueba el par_Cantidad
      IF par_Cantidad IS NULL 
         THEN  RAISE error;
         RSP :='La cantidad no puede ser nula';
      END IF;

 
      -- Hay que comprobar que el NIF exista en la base de datos y el id_cartera.
         BEGIN
 	         SELECT id_cartera INTO v_cartera
                 FROM cartera
                 WHERE NIF = par_NIF;
         EXCEPTION
                WHEN NO_DATA_FOUND THEN
                  RSP := 'ERROR: Los datos NIF e id_cartera no existen en la base de datos';
                  RAISE error;
         END;


     -- Hay que comprobar que el NIF tenga tickers en su cartera para vender.

              IF par_Tipo_orden='VENTA' THEN 
    

                      BEGIN
 	                   SELECT caracc.cantidad INTO v_cantidad
                           FROM cartera_acciones caracc, cartera car 
                           WHERE caracc.id_cartera = car.id_cartera 
                                 and car.NIF=par_NIF
                                 and caracc.ticker=par_ticker
                                 and caracc.fecha_fin is null;
                    EXCEPTION
                          WHEN NO_DATA_FOUND THEN
                          RSP := 'ERROR: El cliente no tiene ese ticker en su cartera de acciones';
                          RAISE error;
                     END;
             END IF;

        -- Se comprueba que si es tipo_orden VENTA, la cartera tenga esa cantidad de tickers para vender.
              IF (par_Tipo_orden='VENTA' AND v_cantidad =< par_Cantidad) THEN
                THEN  RAISE error;
                RSP :='La cantidad de acciones del cliente para vender es: ' || v_cantidad;
              END IF;

       -- Se inserta la secuencia si el origen de la orden de compra/venta es automtica.
              IF par_Tipo_origen='AUTOMATICA' then
                 v_trading:= SECUENCIA_TRADING.nextval;
              END IF;


       -- se inserta en la tabla ORDEN_COMPRA_VENTA la fila con los valores de los parmetros.
      INSERT INTO ORDEN_COMPRA_VENTA (Id_orden,NIF,par_id_trading,Ticker,Tipo_orden,Tipo_origen,Fecha_entrada_broker,
       Estado,Cantidad,Precio_mnimo,Precio_ejecucin,) 
         VALUES (SECUENCIA_ORDEN_COMPRA_VENTA.nextval,par_NIF,v_trading,par_Ticker,par_Tipo_orden,par_Tipo_origen,
                SYSDATE,par_estado,par_Cantidad,par_Precio_minimo,par_Precio_ejecucion');

      -- se inserta e la tabla LOGS lo que se ha ejecutado para comprobarlo
      INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_ORDEN_COMPRA_VENTA.IN_ORDEN_COMPRA_VENTA',PAR_IN,RSP,'OK',v_usuario);

     EXCEPTION

      WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_ORDEN_COMPRA_VENTA.IN_ORDEN_COMPRA_VENTA',PAR_IN,RSP,'ERROR',v_usuario);

         
      WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         Res:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_ORDEN_COMPRA_VENTA.IN_ORDEN_COMPRA_VENTA,PAR_IN,RSP,'ERROR',v_usuario);

  
  END IN_ORDEN_COMPRA_VENTA;


/* Procedimiento para modificar rdenes de alta rdenes de compra/venta, hay que insertar en la tabla ORDEN_COMPRA_VENTA e insertar los valores de los parmetros 
  
   PARAMETROS:
        INPUT: par_Id_orden IN ORDEN_COMPRA_VENTA.Id_orden%TYPE,
               par_Tipo_orden IN ORDEN_COMPRA_VENTA.Tipo_orden%TYPE,
               par_Tipo_origen IN ORDEN_COMPRA_VENTA.Tipo_origen%TYPE,
               par_Fecha_entrada_broker IN ORDEN_COMPRA_VENTA.Fecha_entrada_broker%TYPE,
               par_Estado IN ORDEN_COMPRA_VENTA.Estado%TYPE, 
               par_id_trading IN ORDEN_COMPRA_VENTA.id_trading%TYPE,
               par_Fecha_ejec_mercado IN ORDEN_COMPRA_VENTA.Fecha_ejec_mercado%TYPE,
               par_Cantidad IN ORDEN_COMPRA_VENTA.Cantidad%TYPE,
               par_Precio_minimo IN ORDEN_COMPRA_VENTA.Precio_mnimo%TYPE,
               par_Precio_ejecucion IN ORDEN_COMPRA_VENTA.Precio_ejecucin%TYPE)


        OUTPUT:  RSP 
   PRECONDICION: El valor de los parmetros es correcto.
   POSTCONDICION: Se modifica la rden de compra/venta
   EXCEPTION:   'ERROR: 'El Id_orden no puede ser nulo';
                'ERROR: (SQLERRM)'
*/



PROCEDURE UP_ORDEN_COMPRA_VENTA (par_Id_orden IN ORDEN_COMPRA_VENTA.Id_orden%TYPE,
                                    par_Tipo_orden IN ORDEN_COMPRA_VENTA.Tipo_orden%TYPE,
                                    par_Tipo_origen IN ORDEN_COMPRA_VENTA.Tipo_origen%TYPE,
                                    par_Fecha_entrada_broker IN ORDEN_COMPRA_VENTA.Fecha_entrada_broker%TYPE,
                                    par_Estado IN ORDEN_COMPRA_VENTA.Estado%TYPE, 
                                    par_Fecha_ejec_mercado IN ORDEN_COMPRA_VENTA.Fecha_ejec_mercado%TYPE,
                                    par_Cantidad IN ORDEN_COMPRA_VENTA.Cantidad%TYPE,
                                    par_Precio_minimo IN ORDEN_COMPRA_VENTA.Precio_mnimo%TYPE,
                                    par_Precio_ejecucion IN ORDEN_COMPRA_VENTA.Precio_ejecucin%TYPE);

     error         EXCEPTION;
     numero        NUMBER;
     mensaje       VARCHAR2(200);
     PAR_IN        VARCHAR2(500);
     RSP           VARCHAR2(500);
   --  v_cartera     CARTERA.NIF%TYPE;
    -- v_cantidad    NUMBER;
     v_usuario     VARCHAR2(30);

     BEGIN
 
         SELECT USER into v_usuario
         FROM DUAL;


      PAR_IN := par_Id_orden || ' ' || par_Tipo_orden || ' ' || par_Tipo_origen || ' ' || par_Fecha_entrada_broker
|| ' ' || par_Estado || ' ' || par_Fecha_ejec_mercado || ' ' || par_Cantidad || ' ' || par_Precio_mnimo;
  
       -- se considera que las columnas que se pueden modificar en una orden de compra/venta que est de alta en la tabla
       -- ORDEN_COMPRA_VENTA son: par_Tipo_orden, par_Fecha_entrada_broker, par_Estado, par_Fecha_ejec_mercado,
       -- par_Cantidad, par_Precio_mnimo, par_Precio_ejecucin 


     /* Hay que comprobar que los parmetros obligatorios de entrada sean correctos: */

     -- se comprueba el par_Id_orden
      IF par_Id_orden IS NULL 
      THEN  RAISE error;
        RSP :='El Id_orden no puede ser nulo';
      END IF;
    
     

          -- se actualiza en la tabla ORDEN_COMPRA_VENTA los valores de los parmetros.
         UPDATE ORDEN_COMPRA_VENTA SET Tipo_orden=par_Tipo_orden, Tipo_origen= par_Tipo_origen,
                Fecha_entrada_broker=par_Fecha_entrada_broker, Estado=par_Estado,Fecha_ejec_mercado=par_Fecha_ejec_mercado,
                Cantidad=par_Cantidad, Precio_mnimo=par_Precio_minimo, Precio_ejecucion=par_Precio_ejecucion
         WHERE id_orden=par_id_orden;

     
           -- se inserta en la tabla LOGS para comprobar lo que se ejecuta.
          
      -- se inserta e la tabla LOGS lo que se ha ejecutado para comprobarlo
      INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_ORDEN_COMPRA_VENTA.UP_ORDEN_COMPRA_VENTA',PAR_IN,RSP,'OK',v_usuario);


     EXCEPTION

        WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_ORDEN_COMPRA_VENTA.UP_ORDEN_COMPRA_VENTA',PAR_IN,RSP,'ERROR',v_usuario);

         
      WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         Res:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_ORDEN_COMPRA_VENTA.UP_ORDEN_COMPRA_VENTA,PAR_IN,RSP,'ERROR',v_usuario);

     END;




/* Procedimiento para eliminar rdenes de alta rdenes de compra/venta, hay que actualizar el valor del estado de la tabla ORDEN_COMPRA_VENTA 
  
   PARAMETROS:
        INPUT: par_Id_orden IN ORDEN_COMPRA_VENTA.Id_orden%TYPE);


        OUTPUT:  RSP 
   PRECONDICION: El valor de los parmetros es correcto.
   POSTCONDICION: Se modifica el estado de la rden de compra/venta a cancelada.
   EXCEPTION:   'ERROR: El id_orden no puede ser nulo';
                'ERROR: (SQLERRM)'
*/


  
PROCEDURE DEL_ORDEN_COMPRA_VENTA (par_Id_orden IN ORDEN_COMPRA_VENTA.Id_orden%TYPE);


  BEGIN

   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   RSP           VARCHAR2(500);
   v_NIF         CARTERA.NIF%TYPE;
   v_id_cartera  CARTERA.id_cartera%TYPE;
   v_fecha_baja  CARTERA.id_fecha_baja%TYPE;
   error         EXCEPTION;
   v_usuario     VARCHAR2(30);

         SELECT USER into v_usuario
         FROM DUAL;

     PAR_IN := par_NIF || ' ' || par_fecha;

  -- Hay que comprobar que los parmetros de entrada sean correctos: 
   
     -- se comprueba que el Id_orden no sea nulo.
     RSP := 'ERROR: El id_orden no puede ser nulo';
     IF par_Id_orden IS NULL THEN  RAISE error; END IF;

-- Hay que comprobar que Id_orden exista en la base de datos: 
    
            BEGIN
 	         SELECT ticker INTO v_id_orden
                 FROM ORDEN_COMPRA_VENTA
                 WHERE id_orden=par_Id_orden;
                 

            EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                  RSP := 'ERROR: No existe el id_orden ' || par_Id_orden ;
                  RAISE error;
            END;
              
              -- se cambia el estado a CANCELADA con el id_orden que corresponda al valor del parmetro. 
             UPDATE ORDEN_COMPRA_VENTA SET estado='CANCELADA' WHERE id_orden=par_Id_orden;
        
              -- se inserta en la tabla LOGS para comprobar lo que se ejecuta.
             INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_ORDEN_COMPRA_VENTA.DEL_ORDEN_COMPRA_VENTA',PAR_IN,RSP,'OK',v_usuario);

EXCEPTION

     WHEN error THEN

         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado, descripcion) 
         VALUES(SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_ORDEN_COMPRA_VENTA.UP_ORDEN_COMPRA_VENTA',PAR_IN,RSP,'ERROR',v_usuario);

         
      WHEN OTHERS THEN
         numero := SQLCODE;
         mensaje := SUBSTR(SQLERRM, 1, 200); 

         Res:= 'ERROR '||TO_CHAR(numero)||': '||mensaje;


          INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
          VALUES (SECUENCIA_LOGS.nextval,SYSDATE,'PAQ_ORDEN_COMPRA_VENTA.UP_ORDEN_COMPRA_VENTA,PAR_IN,RSP,'ERROR',v_usuario);


   END;


END PAQ_ORDEN_COMPRA_VENTA;