/* Descripcin: Listado de las diez acciones ms adquiridas por los sistemas automticas, 
	ordenados de forma decreciente por su volumen.
   Parmetros entrada: No es necesario.
 
   Parmetros salida: Identificador de la accin (ticker), nombre completo de la accin (nombre)
	
   Tablas: ACCIN, ORDEN_COMPRA_VENTA

   INstrucciones de ejecucin: EXECUTE PROC_ACCIONES_AUT ();


 */

create or replace PROCEDURE PROC_ACCIONES_AUT AS
                   -- en el cursor se almacena el nombre de la accin y la cantidad
         CURSOR C1 IS SELECT A.nombre, SUM(CV.cantidad) cantidad
                             FROM ORDEN_COMPRA_VENTA CV, ACCION A
                             WHERE CV.ticker=A.ticker
                                   AND CV.Tipo_origen='AUTOMATICA' 
                                   AND CV.estado = 'EJECUTADA'
                                   AND ROWNUM <=10
                              GROUP BY A.nombre
                              ORDER BY cantidad DESC;
                       
           err_num       NUMBER(6,2);
           err_msg       VARCHAR2(200);
           res      VARCHAR2(200);
           RSP           VARCHAR2(500);
           PAR_IN        VARCHAR2(30);
           v_usuario     VARCHAR2(30); 
 
	  BEGIN

        SELECT USER into v_usuario
        FROM DUAL;

         DBMS_OUTPUT.PUT_LINE('TICKER, NOMBRE COMPLETO');
             
		FOR fila IN C1 LOOP
                          
                           -- se visualizan los datos en pantalla
			  DBMS_OUTPUT.PUT_LINE(fila.nombre ||' '||fila.cantidad);
		END LOOP;

           -- Se actualiza la tabla LOGS para saber lo que se ha ejecutado. 
         INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
         VALUES (secuencia_logs.nextval,SYSDATE, 'PROC_ACCIONES_AUT',PAR_IN, 'RSP: '||RSP,'OK',v_usuario);
         COMMIT;

         EXCEPTION
		WHEN NO_DATA_FOUND THEN
                       
              err_num := SQLCODE;
              err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

                INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
                VALUES (secuencia_logs.nextval,SYSDATE,'PROC_ACCIONES_AUT','PAR_IN',RSP,'ERROR',v_usuario);
               

        WHEN OTHERS THEN
              err_num := SQLCODE;
              err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

                INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
                VALUES (secuencia_logs.nextval,SYSDATE,'PROC_ACCIONES_AUT','PAR_IN',RSP,'ERROR',v_usuario);
            
END;