/* Descripcin: Listado de todas las acciones sobre las cuales se han realizado alguna operacin.
   
   Parmetros entrada: Ao
   
   Parmetros salida: Identificador de la accin (ticker), nombre completo de la accin (nombre), 
	nmero de las operaciones de compra ejecutadas durante ese ao, 
	suma total en euros de las rdenes de compra ejecutadas durante el ao sobre la accin, 
	valor medio en euros de las rdenes de compra ejecutadas durante ese ao sobre la accin, 
	nmero de las operaciones de venta ejecutadas durante ese ao sobre la accin, 
	suma total en euros de las rdenes de venta ejecutadas sobre la accin, 
	revalorizacin de la accin en porcentaje desde el uno de enero del ao indicado.
   Tablas:  ACCIN, ORDEN_COMPRA_VENTA, LOGS

   INstrucciones de ejecucin: EXECUTE PROC_ACCIONES_OPER ('2015'); 

*/

create or replace PROCEDURE proc_acciones_oper (par_anio VARCHAR2)  AS

    -- Se realiza el cursor con los datos de las acciones de compra y de venta.
          CURSOR C1 IS SELECT * FROM (select cv.ticker TICKER, acc.nombre NOMBRE, 
                                count(cv.id_orden) CONTAR,sum(cv.precio_ejecucin) SUMA, 
                                 avg(cv.precio_ejecucin) MEDIA, cv.tipo_orden ORDEN 
                                 from ACCION acc, ORDEN_COMPRA_VENTA cv
                                 where acc.ticker=cv.ticker
                                 and TO_CHAR(cv.Fecha_ejec_mercado,'YYYY')=par_anio
                                 and cv.tipo_orden='COMPRA' and cv.estado='EJECUTADA'
   		                           group by cv.ticker, acc.nombre,cv.tipo_orden 
                                UNION
                                select cv.ticker TICKER, acc.nombre NOMBRE,
                                count(cv.id_orden) CONTAR, sum(cv.precio_ejecucin) SUMA,
                                0, cv.tipo_orden ORDEN
                                from ACCION acc, ORDEN_COMPRA_VENTA cv
                                where acc.ticker=cv.ticker
                                and TO_CHAR(cv.Fecha_ejec_mercado,'YYYY')=par_anio
                                and cv.tipo_orden='VENTA' and cv.estado='EJECUTADA'
   		                          group by cv.ticker, acc.nombre, 0,cv.tipo_orden )
                                order by 6;
          v_precioen NUMBER(20,6);	
          v_precio NUMBER(20,6);
          v_diferencia NUMBER(20,6);
          v_reval NUMBER(20,6);
          RSP           VARCHAR2(500);
          PAR_IN        VARCHAR2(200);
          err_num       NUMBER;
          err_msg       VARCHAR2(300);
          FECHA_AYER    VARCHAR2(30);
          v_usuario     VARCHAR2(30); 
          
                   
	  BEGIN
           PAR_IN := par_anio;
           
           SELECT USER into v_usuario
           FROM DUAL;
           
           SELECT TO_CHAR(TO_DATE(SYSDATE,'DD/MM/YYYY') - 1) INTO FECHA_AYER
           FROM DUAL;
           DBMS_OUTPUT.PUT_LINE('TICKER, NOMBRE,N,SUMA, MEDIA, %REVALOR');
        
		FOR fila IN C1 LOOP
    -- Para la revalorizacin de la accin en % desde el uno de enero:  
                            -- consulto el precio de la accin el uno de enero
                            select precio_euro into v_precioen
                            from precio_accion
                            where ticker = fila.ticker
                            and fecha = '01/01/2015';
                            
                            -- consulto el precio de la accin el da anterior porque considero a cierre de mercado
                            select precio_euro into v_precio
                            from precio_accion
                            where ticker = 'SAB'
                            and TO_CHAR(TO_DATE(fecha,'DD/MM/YYYY'))= TO_CHAR(TO_DATE(FECHA_AYER,'DD/MM/YYYY'));

                            
                            -- hacer diferencia para calcular la revalorizacin
                              v_diferencia := v_precio - v_precioen;
                              v_reval := (100 * v_diferencia)/v_precioen;
                 
        EXIT WHEN C1%NOTFOUND; 
        
        DBMS_OUTPUT.PUT_LINE(fila.TICKER||' '||fila.NOMBRE||' '||fila.CONTAR||' '|| fila.SUMA||' '|| fila.MEDIA ||' '|| fila.ORDEN ||' ' || v_reval ||' '|| '%');
    
    END LOOP;
    
    -- Se actualiza la tabla LOGS para saber lo que se ha ejecutado. 
               INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion)
               VALUES (secuencia_logs.nextval,SYSDATE, 'PROC_ACC_OPER', PAR_IN, 'RSP: '||RSP,'OK',v_usuario);
                
  EXCEPTION
		WHEN NO_DATA_FOUND THEN
                       
              err_num := SQLCODE;
              err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

                INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
                VALUES (secuencia_logs.nextval,SYSDATE,'PROC_ACCIONES_OPER',PAR_IN,RSP,'ERROR',v_usuario);
               

        WHEN OTHERS THEN
              err_num := SQLCODE;
              err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

                INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
                VALUES (secuencia_logs.nextval,SYSDATE,'PROC_ACCIONES_OPER',PAR_IN,RSP,'ERROR',v_usuario);
         END;