/* Descripcin: Listado de todos los clientes que han realizado operaciones durante este ao.
   
   Parmetros entrada: Ao.
    
   Parmetros salida: identificador del cliente (NIF), nombre del cliente (Nombre, apellido1, apellido2), 
	nmero de rdenes de compra cargadas durante ese ao,
        nmero de rdenes de venta cargadas durante ese ao, 
	nmero de rdenes de compra ejecutadas durante ese ao,
        sumatorio en euros de todas las rdenes de compra ejecutadas durante ese ao,
	valor medio en euros de cada operacin de compra ejecutada por el cliente durante ese ao,
        nmero de rdenes de venta ejecutadas durante ese ao, 
	sumatorio en euros de todas las rdenes de venta ejecutadas durante ese ao, 
        valor medio en euros de cada operacin de venta ejecutada por el cliente durante ese ao,
	nmero de rdenes de compra canceladas durante ese ao, 
        nmero de rdenes de venta canceladas durante ese ao.
   
    Tablas: CLIENTE, ORDEN_COMPRA_VENTA 

    Instrucciones: EXECUTE proc_clioper ('2015');

*/

  
create or replace PROCEDURE proc_clioper (par_anio VARCHAR2) AS
    
CURSOR C1 IS SELECT *
  FROM(
    SELECT orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,COUNT (orden.id_orden) cuenta,
    SUM(precio_ejecucin) suma,AVG(precio_ejecucin) media,'COMPRA' columna
    FROM ORDEN_COMPRA_VENTA orden,CLIENTE cli
    WHERE orden.NIF=cli.NIF
    AND TO_CHAR(orden.Fecha_ejec_mercado,'YYYY')=par_anio
    AND  orden.tipo_orden='COMPRA'
    AND orden.estado='EJECUTADA'
    GROUP BY orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,'COMPRA'
UNION
SELECT orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,COUNT (orden.id_orden) cuenta,
    SUM(precio_ejecucin) suma,AVG(precio_ejecucin) media,'VENTA' columna
    FROM ORDEN_COMPRA_VENTA orden,CLIENTE cli
    WHERE orden.NIF=cli.NIF
    AND TO_CHAR(orden.Fecha_ejec_mercado,'YYYY')=par_anio
    AND  orden.tipo_orden='VENTA'
    AND orden.estado='EJECUTADA'
    GROUP BY orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,'VENTA'
UNION
SELECT orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,COUNT (orden.id_orden) cuenta,
    SUM(precio_ejecucin) suma,AVG(precio_ejecucin) media,'COMPRA_CANCELADA' columna
    FROM ORDEN_COMPRA_VENTA orden,CLIENTE cli
    WHERE orden.NIF=cli.NIF
    AND TO_CHAR(orden.Fecha_entrada_broker,'YYYY')=par_anio
    AND  orden.tipo_orden='COMPRA'
    AND orden.estado='CANCELADA'
    GROUP BY orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,'COMPRA_CANCELADA'
  UNION
  SELECT orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,COUNT (orden.id_orden) cuenta,
    SUM(precio_ejecucin) suma,AVG(precio_ejecucin) media,'VENTA_CANCELADA' columna
    FROM ORDEN_COMPRA_VENTA orden,CLIENTE cli
    WHERE orden.NIF=cli.NIF
    AND TO_CHAR(orden.Fecha_entrada_broker,'YYYY')=par_anio
    AND  orden.tipo_orden='VENTA'
    AND orden.estado='CANCELADA'
    GROUP BY orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,'VENTA_CANCELADA'
    UNION
    SELECT orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,COUNT (orden.id_orden) cuenta,
    SUM(precio_ejecucin) suma,AVG(precio_ejecucin) media,'COMPRAS_CARGADAS' columna
    FROM ORDEN_COMPRA_VENTA orden,CLIENTE cli
    WHERE orden.NIF=cli.NIF
    AND TO_CHAR(orden.Fecha_entrada_broker,'YYYY')=par_anio
    AND  orden.tipo_orden='COMPRA'
    GROUP BY orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,'COMPRAS_CARGADAS'
    UNION
    SELECT orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,COUNT (orden.id_orden) cuenta,
    SUM(precio_ejecucin) suma,AVG(precio_ejecucin) media,'VENTAS_CARGADAS' columna
    FROM ORDEN_COMPRA_VENTA orden,CLIENTE cli
    WHERE orden.NIF=cli.NIF
    AND TO_CHAR(orden.Fecha_entrada_broker,'YYYY')=par_anio
    AND  orden.tipo_orden='VENTA'
    GROUP BY orden.NIF, cli.Nombre, cli.apellido1, cli.apellido2,'VENTAS_CARGADAS'
  )
ORDER BY 1;
			     
    
      RSP           VARCHAR2(500);
      PAR_IN        VARCHAR2(200);
      err_num       NUMBER;
      err_msg       VARCHAR2(300);
      v_usuario     VARCHAR2(30); 
               
BEGIN
           PAR_IN := par_anio;
           SELECT USER into v_usuario
           FROM DUAL;
                 
       DBMS_OUTPUT.PUT_LINE('NIF NOMBRE APELLIDO1 APELLIDO2 CUENTA SUMA MEDIA TIPO');
		FOR fila IN C1 LOOP
          
    DBMS_OUTPUT.PUT_LINE(fila.NIF || '' || fila.Nombre || ' ' || fila.apellido1 || ' ' || fila.apellido2
    || ' ' || fila.cuenta || ' ' || fila.suma || ' ' || fila.media || ' ' || fila.columna);
                                    
    END LOOP;
          -- Se actualiza la tabla LOGS para saber lo que se ha ejecutado. 
               INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion)
               VALUES (secuencia_logs.nextval,SYSDATE, 'PROC_CLIOPER', PAR_IN, 'RSP: '||RSP,'OK',v_usuario);
                
            
 EXCEPTION
       WHEN NO_DATA_FOUND THEN
                       
              err_num := SQLCODE;
              err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

                INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
                VALUES (secuencia_logs.nextval,SYSDATE,'PROC_CLIOPER',PAR_IN,RSP,'ERROR',v_usuario);
               

        WHEN OTHERS THEN
              err_num := SQLCODE;
              err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

                INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
                VALUES (secuencia_logs.nextval,SYSDATE,'PROC_CLIOPER',PAR_IN,RSP,'ERROR',v_usuario);
                
END;