/* TRIGGER PARA ACTUALIZAR LA TABLA EST_CARTERA_MAYOR DEL MDULO DE ESTADSTICAS 
   
    Trigger con la cartera de acciones con un valor ms grande, y su valor en euros de su cartera.
    Se dispara cuando se producen acciones en la tabla PRECIOS_ACCIONES.

    Se considera que las acciones del cliente estn de alta en el da SYSDATE por lo tanto
    tengo que considerar aquellas que estn de alta en ese da por lo tanto fecha_hasta is null
    o justo ese mismo da causa baja;

    La hora de apertura de la bolsa es a las 09:00 horas y la hora de cierre es a las 17:30 horas.
    Se actualiza el valor de las acciones en PRECIOS_ACCIONES y se tiene que recalcular todos los das.
     
    En la tabla de LOG quiero guardar lo que ejecuto y si sucede error. 
   
    Para controlar que sea la cartera con el valor ms grande puedo utilizar la SELECT o el FOR

Para estadsticas: 
                    INSERT INTO PRECIO_ACCION (Ticker,Fecha,Precio_euro,Precio_dolar) 
                     VALUES ('GRFS','05/06/2015',36.80,39.97);
                    INSERT INTO PRECIO_ACCION (Ticker,Fecha,Precio_euro,Precio_dolar) 
                     VALUES ('SAB','05/06/2015',2.49,2.53);
                    COMMIT;
                    SELECT * FROM EST_CARTERA_MAYOR;

 */

create or replace TRIGGER TR_MAYOR 
 AFTER INSERT OR UPDATE OR DELETE ON PRECIO_ACCION
  DECLARE          -- Se define el cursor que almacena las carteras activas y su valor

   CURSOR C1 IS SELECT car.NIF, SUM(caracc.cantidad*pr.precio_euro) valor
                FROM CARTERA car, CARTERA_ACCION caracc, ACCION acc, PRECIO_ACCION pr
                WHERE car.id_cartera=caracc.id_cartera
                AND caracc.ticker=acc.ticker  
                AND caracc.fecha_inicio < = SYSDATE 
                AND (caracc.FECHA_FIN is null OR caracc.FECHA_FIN >= SYSDATE)
                AND acc.ticker=pr.ticker
                AND pr.FECHA=TO_CHAR(SYSDATE,'DD/MM/YYYY') 
                GROUP BY car.NIF
                ORDER BY valor DESC;
   
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   RSP           VARCHAR2(500);
   PAR_IN        VARCHAR2(30);
   v_usuario     VARCHAR2(30);   
   v_valor       number(10,2):=0;
   v_NIF         VARCHAR2(10);

 BEGIN

     SELECT USER into v_usuario
     FROM DUAL;
       
    FOR fila IN C1 LOOP
                IF v_valor < fila.valor THEN
                  v_valor:= fila.valor;
                   v_NIF := fila.NIF;
                 END IF;
                     EXIT WHEN C1%NOTFOUND;   
    END LOOP;

               -- Se actualiza la tabla del mdulo de estadsticas EST_CARTERA_MAYOR con los datos el cursor.
              INSERT INTO EST_CARTERA_MAYOR (ID,NIF,valor,fecha) 
                VALUES (SECUENCIA_EST_MAYOR.nextval,v_NIF,v_valor,SYSDATE);
                             
               -- Se actualiza la tabla LOGS para saber lo que se ha ejecutado. 
               INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion)
               VALUES (secuencia_logs.nextval,SYSDATE, 'TR_MAYOR', 'PAR_IN', 'RSP: '||RSP,'OK',v_usuario);
 
 EXCEPTION

        WHEN NO_DATA_FOUND THEN
                       
              err_num := SQLCODE;
              err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

                INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
                VALUES (secuencia_logs.nextval,SYSDATE,'TR_MAYOR','PAR_IN',RSP,'ERROR',v_usuario);
               

        WHEN OTHERS THEN
              err_num := SQLCODE;
              err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

                INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion) 
                VALUES (secuencia_logs.nextval,SYSDATE,'TR_MAYOR','PAR_IN',RSP,'ERROR',v_usuario);
            

 END;