
/* TRIGGER PARA ACTUALIZAR LA TABLA EST_PENDIENTES  

La tabla EST_PENDIENTES contiene las acciones disponible dentro del broker: la suma de las acciones que 
estn en la tabla ORDEN_COMPRA_VENTA que no estn ejecutadas ni canceladas y el nmero de clientes que
la tienen actualmente en su cartera.

Dado un identificador de la accin hay que visualizar el nmero de acciones disponibles en venta y 
  el nmero de clientes que las tienen en su cartera de forma activa.

En la tabla de LOGS quiero guardar lo que ejecuto y si sucede error. 


Para estadsticas: UPDATE ORDEN_COMPRA_VENTA SET Estado= 'PENDIENTE',Fecha_ejec_mercado=null WHERE Id_orden='OR006';
                   COMMIT;
                    SELECT * FROM EST_PENDIENTES;
                  
                   EXECUTE PR_PENDIENTES('SAB') ;
  
*/        

create or replace TRIGGER TR_PENDIENTES
 AFTER UPDATE OR INSERT OR DELETE ON ORDEN_COMPRA_VENTA
        
	DECLARE
         -- se almacena en el cursor la cantidad de acciones de venta disponibles.
	CURSOR C1 IS SELECT ord.ticker, sum (ord.cantidad) cant 
                     FROM ORDEN_COMPRA_VENTA ord
                     WHERE ord.estado = 'PENDIENTE'
                     AND ord.tipo_orden='VENTA'
                     GROUP BY ord.ticker;
	      

   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   RSP           VARCHAR2(500);
   v_nif         CARTERA.NIF%TYPE;
   v_usuario     VARCHAR2(30);
     
	BEGIN
  
         SELECT USER into v_usuario
         FROM DUAL;
             
             FOR fila1 IN C1 LOOP
               	     
	        EXIT WHEN C1%NOTFOUND;                     
                 -- Nmero de clientes que tienen la accin en su cartera de forma activa
                SELECT count(cart.NIF) into v_nif 
                FROM CARTERA cart, CARTERA_ACCION caracc
		            WHERE cart.id_cartera=caracc.id_cartera
		            AND caracc.fecha_fin is null
		            AND caracc.ticker=fila1.ticker; 
                          
                 -- Se inserta en la tabla de estadsticas las acciones disponibles dentro del broker                      
                   INSERT INTO EST_PENDIENTES (id,Ticker,acciones_dispo_broker,num_clientes,fecha) 
                   VALUES (secuencia_est_pendientes.nextval,fila1.ticker,fila1.cant,v_nif,SYSDATE);
	     END LOOP;
            -- Se inserta en la tabla LOGS para saber lo que se ejecuta.
            INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado,descripcion)
               VALUES (secuencia_logs.nextval,SYSDATE, 'TR_PENDIENTES', 'PAR_IN', 'RSP: '||RSP,'OK',v_usuario);
 
EXCEPTION    
          
        WHEN NO_DATA_FOUND THEN
              err_num := SQLCODE;
              err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

                INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado) 
                VALUES (secuencia_logs.nextval,SYSDATE,'TR_PENDIENTES','PAR_IN',RSP,'ERROR');
                 COMMIT;

        WHEN OTHERS THEN
              err_num := SQLCODE;
              err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

                INSERT INTO LOGS (Id_log,Fecha_ejecucin,Func_proc,Par_entrada,Par_salida,Estado) 
                VALUES (secuencia_logs.nextval,SYSDATE,'TR_PENDIENTES','PAR_IN',RSP,'ERROR');
               COMMIT;
	END;